/*
	ReInterpolate420 plugin for AviSynth -- Corrects point sampling of DV MainConcept codec

	Copyright (C) 2004 Wilbert Dijkhof
  
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.
	 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
		
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	The author can be contacted at:
	Wilbert Dijkhof
	wilbertdijkhof@yahoo.com

	v3 22 July 2006 (by Fizick: fizick@avisynth.org.ru) - 
			color tint decreasing in progressive mode, 
			speed increasing in both modes by integer processing 
	v1.1 24rd September 2004 - interlaced support
	v1.0 23rd September 2004 - 1st release

	See more info at http://forum.doom9.org/showthread.php?t=82787
*/

#include "ReInterpolate420.h"


ReInterpolate420::ReInterpolate420(PClip _child, bool _interlaced, IScriptEnvironment* env) : 
	GenericVideoFilter(_child), interlaced(_interlaced) {
	// parameter checking goes in here
	if (!vi.IsYUY2())
		env->ThrowError("ReInterpolate420: input to filter must be in YUY2");
	}

ReInterpolate420::~ReInterpolate420() {
}
	
PVideoFrame __stdcall ReInterpolate420::GetFrame(int n, IScriptEnvironment* env) {

	PVideoFrame src = child->GetFrame(n, env);

	env->MakeWritable(&src);

	//unsigned char* srcp = src->GetWritePtr();
	BYTE* src_plane = src->GetWritePtr();
	const int src_pitch = src->GetPitch();
	const int src_width = src->GetRowSize();
	const int src_height = src->GetHeight();


	if (!interlaced) {
		src_plane += 2*src_pitch; // h=0 first two and last two lines unaltered
		for (int h=2; h < src_height-4; h+=4) {
			for (int w = 0; w < src_width; w+=4) {
				// V3 = 0.666*V2 + 0.333*V5, U3 unaltered
//				src_plane[w+3] = int(0.666*src_plane[w+3-src_pitch] + 0.333*src_plane[w+3+2*src_pitch]); // V value
				src_plane[w+3] = ((2*src_plane[w+3-src_pitch] + src_plane[w+3+2*src_pitch])*341+682)>>10; // V value
				// V4 = 0.333*V2 + 0.666*V5, U4 unaltered
//				src_plane[w+3+src_pitch] = int(0.333*src_plane[w+3-src_pitch] + 0.666*src_plane[w+3+2*src_pitch]); // V value
				src_plane[w+3+src_pitch] = ((src_plane[w+3-src_pitch] + 2*src_plane[w+3+2*src_pitch])*341+682)>>10; // V value
				// U5 = 0.666*U4 + 0.333*U7, V5 unaltered
//				src_plane[w+1+2*src_pitch] = int(0.666*src_plane[w+1+src_pitch] + 0.333*src_plane[w+1+4*src_pitch]); // U value
				src_plane[w+1+2*src_pitch] = ((2*src_plane[w+1+src_pitch] + src_plane[w+1+4*src_pitch])*341+682)>>10; // U value
				// U6 = 0.333*U4 + 0.666*U7, V6 unaltered
//				src_plane[w+1+3*src_pitch] = int(0.333*src_plane[w+1+src_pitch] + 0.666*src_plane[w+1+4*src_pitch]); // U value
				src_plane[w+1+3*src_pitch] = ((src_plane[w+1+src_pitch] + 2*src_plane[w+1+4*src_pitch])*341+682)>>10; // U value

			}
			src_plane += 4*src_pitch;
		}
	} else { // interlaced
		src_plane += 2*src_pitch; // h=0 first two and last two lines unaltered
		for (int h=2; h < src_height-4; h+=4) {
			for (int w = 0; w < src_width; w+=4) {
				// V3 = 0.5*V1 + 0.5*V5, U3 unaltered
//				src_plane[w+3] = int(0.5*src_plane[w+3-2*src_pitch] + 0.5*src_plane[w+3+2*src_pitch]); // V value
				src_plane[w+3] = (src_plane[w+3-2*src_pitch] + src_plane[w+3+2*src_pitch] +1)>>1; // V value
				// V4 = 0.5*V2 + 0.5*V6, U4 unaltered
//				src_plane[w+3+src_pitch] = int(0.5*src_plane[w+3-src_pitch] + 0.5*src_plane[w+3+3*src_pitch]); // V value
				src_plane[w+3+src_pitch] = (src_plane[w+3-src_pitch] + src_plane[w+3+3*src_pitch] +1)>>1; // V value
				// U5 = 0.5*U3 + 0.5*U7, V5 unaltered
//				src_plane[w+1+2*src_pitch] = int(0.5*src_plane[w+1] + 0.5*src_plane[w+1+4*src_pitch]); // U value
				src_plane[w+1+2*src_pitch] = (src_plane[w+1] + src_plane[w+1+4*src_pitch] +1)>>1; // U value
				// U6 = 0.5*U4 + 0.5*U8, V6 unaltered
//				src_plane[w+1+3*src_pitch] = int(0.5*src_plane[w+1+src_pitch] + 0.5*src_plane[w+1+5*src_pitch]); // U value
				src_plane[w+1+3*src_pitch] = (src_plane[w+1+src_pitch] + src_plane[w+1+5*src_pitch] +1)>>1; // U value

			}
			src_plane += 4*src_pitch;
		}
	}

	_asm emms;
	return src;
}

AVSValue __cdecl Create_ReInterpolate420(AVSValue args, void* user_data, IScriptEnvironment* env) {
    return new ReInterpolate420(args[0].AsClip(),
			args[1].AsBool(false),
			env);
}


extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
	env->AddFunction("ReInterpolate420", "c[interlaced]b", Create_ReInterpolate420, 0);
    return "`ReInterpolate420' ReInterpolate420 plugin";
}

